package com.aliyun.iot.ilop.page.deviceadd.search;

import android.content.Context;
import android.text.TextUtils;

import com.aliyun.iot.aep.sdk.apiclient.IoTAPIClient;
import com.aliyun.iot.aep.sdk.apiclient.IoTAPIClientFactory;
import com.aliyun.iot.aep.sdk.apiclient.callback.IoTCallback;
import com.aliyun.iot.aep.sdk.apiclient.callback.IoTResponse;
import com.aliyun.iot.aep.sdk.apiclient.emuns.Scheme;
import com.aliyun.iot.aep.sdk.apiclient.request.IoTRequest;
import com.aliyun.iot.aep.sdk.apiclient.request.IoTRequestBuilder;
import com.aliyun.iot.aep.sdk.log.ALog;
import com.aliyun.iot.aep.sdk.threadpool.ThreadPool;
import com.aliyun.iot.ilop.component.deviceadd.module.Product;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;


public class GlobalBusiness {

    //用于存储全局搜索内容
    public static List<Object> globalProductList = new ArrayList<>();

    private static String TAG = "GlobalBusiness";
    private List<Product> productList;
    private Context context;

    public GlobalBusiness(Context context) {
        this.context = context;
    }

    public void allProductRequest(final GlobalCallBack callBack) {
        IoTRequestBuilder builder = new IoTRequestBuilder()
                .setPath("/home/app/product/query")
                .setScheme(Scheme.HTTPS)
                .setApiVersion("1.0.0")
                .setAuthType("iotAuth")
                .addParam("pageNo", 1)
                .addParam("pageSize", 100);

        final IoTAPIClient client = new IoTAPIClientFactory().getClient();

        client.send(builder.build(), new IoTCallback() {
            @Override
            public void onFailure(IoTRequest request, Exception e) {
                ALog.d(TAG, "onFailure：" + e.toString());
                callBack.onFailed(e.getMessage());
            }

            @Override
            public void onResponse(IoTRequest request, IoTResponse response) {
                if (response.getCode() != 200) {
                    callBack.onFailed(response.getMessage());
                }
                JSONArray items = (JSONArray) response.getData();
                for (int i = 0; i < items.length(); i++) {
                    try {
                        JSONObject jsonObject = items.getJSONObject(i);
                        JSONArray dtoList = jsonObject.optJSONArray("simpleProductDTOList");
                        for (int j = 0; j < dtoList.length(); j++) {
                            JSONObject object = dtoList.getJSONObject(j);
                            String productKey = object.getString("productKey");
                            String productName = object.getString("productName");
                            boolean has = object.has("netType");
                            boolean hasUrl = object.has("productImage");
                            Product product = new Product();
                            if (has) {
                                try {
                                    product.netType = object.getString("netType");
                                } catch (Exception ignored) {

                                }
                            }
                            if (hasUrl) {
                                product.productImage = object.getString("productImage");
                            }
                            product.productKey = productKey;
                            product.name = productName;
                            globalProductList.add(product);
                        }

                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }
                callBack.onSuccess(globalProductList);
            }
        });
    }


    public void searchRequset(String word, final SearchCallBack searchCallBack) {
        if (productList == null) {
            return;
        }
        if (searchCallBack == null) {
            return;
        }

        final List<Product> newProduct = new ArrayList<>();
        if (TextUtils.isEmpty(word)) {
            newProduct.clear();
        } else {
            for (int i = 0; i < productList.size(); i++) {
                if (productList.get(i).name.contains(word) || productList.get(i).name.toLowerCase().contains(word.toLowerCase())) {
                    newProduct.add(productList.get(i));
                }
            }
        }
        ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
            @Override
            public void run() {
                try {
                    searchCallBack.onSearchSuccess(newProduct);
                } catch (Exception ex) {
                    ALog.e(TAG, "exception happen when call listener.onSuccess", ex);
                    ex.printStackTrace();
                }
            }
        });

    }

    public void getAllProduct(List<Product> productList) {
        this.productList = productList;
    }
}
