package com.aliyun.iot.homelink.demo.page.scene.list;

import com.alibaba.fastjson.JSON;
import com.aliyun.iot.homelink.demo.page.scene.AssetsUtils;
import com.aliyun.iot.homelink.demo.page.scene.api.ScenesResponse;
import com.aliyun.iot.homelink.demo.page.scene.entities.Order;

import java.io.IOException;
import java.util.List;

/**
 * @author sinyuk
 * @date 2018/6/14
 * <p>
 * mock presenter for testing only
 */
public class MockSceneListPresenter implements SceneListContract.Presenter {
    private final SceneListContract.View sceneListView;
    private final String groupId;

    MockSceneListPresenter(SceneListContract.View view, String groupId) {
        sceneListView = view;
        this.groupId = groupId;
        view.setPresenter(this);
    }

    @Override
    public void requestBackdropUrl() {
        sceneListView.showBackdrop("");
    }

    @Override
    public void loadScenes(int pageNo) {
        if (1 == pageNo) {
            sceneListView.showRefresh();
        }

        try {
            final ScenesResponse data = JSON.parseObject(
                    AssetsUtils.getInstance().mockResponse("scene_list_get.json"),
                    ScenesResponse.class);
            if (1 == pageNo) {
                sceneListView.initScenes(data);
                sceneListView.hideRefresh();
            } else {
                sceneListView.loadMoreScenes(data);
            }
        } catch (IOException e) {
            String error = e.getMessage();
            if (1 == pageNo) {
                sceneListView.refreshFailed(error);
            } else {
                sceneListView.loadMoreFailed(error);
            }
        }
    }

    @Override
    public void reorder(List<Order> orders) {

    }

    @Override
    public void loadDefaultHouse() {

    }

    @Override
    public void switchHouse(String groupId, int role) {
        loadScenes(1);
    }

    @Override
    public void fire(String sceneId) {
        sceneListView.onFireSucceed(sceneId);
    }

    @Override
    public String groupId() {
        return groupId;
    }

    @Override
    public int role() {
        return 0;
    }

    @Override
    public void start() {
        requestBackdropUrl();
        loadScenes(1);
    }
}
