package com.aliyun.iot.homelink.demo.page.scene.log;

import com.alibaba.fastjson.JSON;
import com.aliyun.iot.homelink.demo.page.scene.AssetsUtils;
import com.aliyun.iot.homelink.demo.page.scene.api.LogsResponse;

import java.io.IOException;

import static com.aliyun.iot.homelink.demo.page.scene.Utils.checkNotNull;

/**
 * @author sinyuk
 * @date 2018/6/15
 */
class MockLogListPresenter implements LogListContract.Presenter {

    private final LogListContract.View logListView;
    private String groupId;

    MockLogListPresenter(LogListContract.View logListView, String groupId) {
        checkNotNull(groupId);
        this.groupId = groupId;
        this.logListView = logListView;
        this.logListView.setPresenter(this);
    }

    @Override
    public void start() {
        loadLogs(1);
    }

    @Override
    public void loadLogs(int pageNo) {
        if (pageNo == 1) {
            logListView.showRefresh();
        }

        try {
            final LogsResponse data = JSON.parseObject(
                    AssetsUtils.getInstance().mockResponse("log_list_get.json"),
                    LogsResponse.class);
            if (1 == pageNo) {
                logListView.initLogs(data);
                logListView.hideRefresh();
            } else {
                logListView.loadMoreLogs(data);
            }
        } catch (IOException e) {
            String error = e.getMessage();
            if (1 == pageNo) {
                logListView.refreshFailed(error);
            } else {
                logListView.loadMoreFailed(error);
            }
        }
    }

    @Override
    public void updateTimestamp(long time) {
        loadLogs(1);
    }

    @Override
    public String groupId() {
        return groupId;
    }
}
