package com.aliyun.iot.homelink.demo.page.scene.details;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.aliyun.iot.homelink.demo.page.scene.R;
import com.aliyun.iot.homelink.demo.page.scene.api.DetailsResponse;
import com.aliyun.iot.homelink.demo.page.scene.widgets.InsetDividerDecoration;
import com.aliyun.iot.link.ui.component.LinkToast;
import com.chad.library.adapter.base.BaseQuickAdapter;

/**
 * @author sinyuk
 * @date 2018/6/19
 */
public class DetailListFragment extends Fragment implements DetailListContract.View {
    private int pageNo = 1;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.details_list_view, container, false);

    }

    private RecyclerView recyclerView;
    private SwipeRefreshLayout swipeRefreshLayout;

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        recyclerView = view.findViewById(R.id.recycleView);
        swipeRefreshLayout = view.findViewById(R.id.refreshLayout);

        setupActionBar(view);
        setupRecyclerView();

        detailListPresenter.start();
    }

    private void setupActionBar(View view) {
        view.findViewById(R.id.back).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (getActivity() != null) {
                    getActivity().finish();
                }
            }
        });
    }


    private DetailAdapter adapter = null;

    private void setupRecyclerView() {
        final LinearLayoutManager layoutManager = new LinearLayoutManager(getContext());
        recyclerView.setLayoutManager(layoutManager);
        //noinspection ConstantConditions
        InsetDividerDecoration decoration = new InsetDividerDecoration(
                getResources().getDimensionPixelSize(R.dimen.divider_height),
                getResources().getDimensionPixelSize(R.dimen.detail_list_item_margin),
                ContextCompat.getColor(getContext(), R.color.dividerColor));
        recyclerView.addItemDecoration(decoration);
        adapter = new DetailAdapter();
        View empty = View.inflate(getContext(), R.layout.empty_layout, null);
        adapter.setEmptyView(empty);
        recyclerView.setHasFixedSize(true);
        recyclerView.setAdapter(adapter);
        adapter.setEnableLoadMore(false);
        adapter.setOnLoadMoreListener(new BaseQuickAdapter.RequestLoadMoreListener() {
            @Override
            public void onLoadMoreRequested() {
                detailListPresenter.loadDetails(pageNo);
            }
        }, recyclerView);

        swipeRefreshLayout.setColorSchemeColors(getResources().getColor(R.color.colorAccent));
        swipeRefreshLayout.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                pageNo = 1;
                detailListPresenter.loadDetails(pageNo);
            }
        });
    }

    @Override
    public void initDetails(DetailsResponse data) {
        pageNo++;
        adapter.setNewData(data.details);
        adapter.setEnableLoadMore(data.pageSize < data.total);
    }

    @Override
    public void loadMoreDetails(DetailsResponse data) {
        if (data.details == null || data.details.isEmpty()) {
            adapter.loadMoreEnd(true);
        } else {
            if (data.total > (adapter.getItemCount() + data.details.size())) {
                adapter.loadMoreComplete();
                pageNo++;
            } else {
                adapter.loadMoreEnd(true);
            }
            adapter.addData(data.details);
        }
    }

    @Override
    public void loadMoreFailed(String message) {
        adapter.loadMoreFail();
    }

    @Override
    public void refreshFailed(String message) {
        swipeRefreshLayout.setRefreshing(false);
        LinkToast.makeText(getContext(), message, Toast.LENGTH_SHORT).show();
    }

    @Override
    public void hideRefresh() {
        swipeRefreshLayout.setRefreshing(false);
    }

    @Override
    public void showRefresh() {
        swipeRefreshLayout.setRefreshing(true);
    }

    private DetailListContract.Presenter detailListPresenter;

    @Override
    public void setPresenter(DetailListContract.Presenter presenter) {
        detailListPresenter = presenter;
    }
}
