package com.aliyun.iot.homelink.demo.page.scene.details;

import android.support.annotation.NonNull;

import com.aliyun.iot.homelink.demo.page.scene.api.DetailsResponse;
import com.aliyun.iot.homelink.demo.page.scene.api.ResponseCallback;
import com.aliyun.iot.homelink.demo.page.scene.data.SceneRepository;

/**
 * @author sinyuk
 * @date 2018/6/19
 */
public class DetailListPresenter implements DetailListContract.Presenter {
    private static final int PAGE_SIZE = 10;
    private final String groupId;
    private final String logId;
    private final String sceneId;
    private final long time;
    private final SceneRepository sceneRepository;
    private final DetailListContract.View detailListView;

    DetailListPresenter(
            @NonNull String groupId,
            @NonNull String logId,
            @NonNull String sceneId,
            long time,
            @NonNull SceneRepository sceneRepository,
            @NonNull DetailListContract.View detailListView) {
        this.groupId = groupId;
        this.logId = logId;
        this.sceneId = sceneId;
        this.time = time;
        this.sceneRepository = sceneRepository;
        this.detailListView = detailListView;
        this.detailListView.setPresenter(this);
    }

    @Override
    public void loadDetails(final int pageNo) {
        if (pageNo == 1) {
            detailListView.showRefresh();
        }
        sceneRepository.requestFailedLogs(groupId, logId, sceneId, pageNo, PAGE_SIZE, time, new ResponseCallback<DetailsResponse>() {
            @Override
            public void onFail(String message) {
                if (1 == pageNo) {
                    detailListView.refreshFailed(message);
                } else {
                    detailListView.loadMoreFailed(message);
                }
            }

            @Override
            public void onSuccess(DetailsResponse data) {
                if (1 == pageNo) {
                    detailListView.hideRefresh();
                    detailListView.initDetails(data);
                } else {
                    detailListView.loadMoreDetails(data);
                }
            }
        });
    }

    @Override
    public String logId() {
        return logId;
    }

    @Override
    public String sceneId() {
        return sceneId;
    }

    @Override
    public long time() {
        return time;
    }

    @Override
    public void start() {
        loadDetails(1);
    }
}
