package com.aliyun.iot.homelink.demo.page.scene.list;

import android.graphics.drawable.Drawable;
import android.support.annotation.LayoutRes;
import android.support.annotation.NonNull;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.aliyun.iot.homelink.demo.page.scene.R;
import com.aliyun.iot.homelink.demo.page.scene.entities.Scene;
import com.aliyun.iot.homelink.demo.page.scene.widgets.CircleView;
import com.bumptech.glide.Glide;
import com.bumptech.glide.RequestBuilder;
import com.bumptech.glide.request.RequestOptions;
import com.chad.library.adapter.base.BaseViewHolder;

/**
 * @author sinyuk
 * @date 2018/6/12
 */
class SceneItemViewHolder extends BaseViewHolder {
    private final RequestBuilder<Drawable> builder;
    private final ImageView icon;
    private final int textPrimary;
    private final int textSecondary;
    private final CircleView circleView;

    public SceneItemViewHolder(View view) {
        super(view);
        icon = view.findViewById(R.id.icon);
        builder = Glide.with(view.getContext())
                .applyDefaultRequestOptions(new RequestOptions().circleCrop())
                .asDrawable();
        circleView = view.findViewById(R.id.circleView);
        textPrimary = ContextCompat.getColor(view.getContext(), R.color.textColorPrimary);
        textSecondary = ContextCompat.getColor(view.getContext(), R.color.textColorSecondary);
    }

    public static final int ON = 1;
    public static final int OFF = 2;

    public void bind(Scene scene) {
        if (scene == null) {
            return;
        }
        setText(R.id.name, scene.name);
        setText(R.id.description, scene.description);
        builder.load(scene.icon).into(icon);
        if (scene.enable && scene.status == ON) {
            circleView.setEnable(true);
            setTextColor(R.id.name, textPrimary);
        } else {
            circleView.setEnable(false);
            setTextColor(R.id.name, textSecondary);
        }
        addOnClickListener(R.id.circleView);
    }
}