package com.aliyun.iot.homelink.demo.page.scene.list;

import com.aliyun.iot.homelink.demo.page.scene.BasePresenter;
import com.aliyun.iot.homelink.demo.page.scene.BaseView;
import com.aliyun.iot.homelink.demo.page.scene.api.ScenesResponse;
import com.aliyun.iot.homelink.demo.page.scene.entities.Order;
import com.aliyun.iot.homelink.demo.page.scene.entities.Scene;

import java.util.List;


/**
 * The interface Scene list contract.
 *
 * @author sinyuk
 * @date 2018 /6/13 <p> This specifies the contract between the view and the presenter.
 */
public interface SceneListContract {
    /**
     * The interface View.
     */
    interface View extends BaseView<Presenter> {

        /**
         * Init scenes.
         *
         * @param data the data
         */
        void initScenes(ScenesResponse data);

        /**
         * Load more scenes.
         *
         * @param data the data
         */
        void loadMoreScenes(ScenesResponse data);


        /**
         * Hide load more.
         *
         * @param message the message
         */
        void loadMoreFailed(String message);

        /**
         * Refresh failed.
         *
         * @param message the message
         */
        void refreshFailed(String message);

        /**
         * Hide refresh.
         */
        void hideRefresh();

        /**
         * Show refresh.
         */
        void showRefresh();

        /**
         * Show backdrop.
         *
         * @param url the url
         */
        void showBackdrop(String url);

        /**
         * Reorder succeed.
         */
        void reorderSucceed();

        /**
         * Reorder failed.
         *
         * @param message the message
         */
        void reorderFailed(String message);

        /**
         * On fire failed.
         *
         * @param sceneId the scene id
         * @param message the message
         */
        void onFireFailed(String sceneId, String message);

        /**
         * On fire succeed.
         *
         * @param sceneId the scene id
         */
        void onFireSucceed(String sceneId);

        /**
         * Load house failed.
         *
         * @param message the message
         */
        void loadHouseFailed(String message);

        void updateRole(int role);
    }

    /**
     * The interface Presenter.
     */
    interface Presenter extends BasePresenter {

        /**
         * Request backdrop url.
         */
        void requestBackdropUrl();

        /**
         * Load scenes.
         *
         * @param pageNo the page no
         */
        void loadScenes(int pageNo);

        /**
         * reorder.
         *
         * @param orders the orders
         */
        void reorder(List<Order> orders);

        /**
         * Load default house.
         */
        void loadDefaultHouse();

        /**
         * Switch house.
         *
         * @param groupId the group id
         */
        void switchHouse(String groupId, int role);

        /**
         * Fire.
         *
         * @param sceneId the scene id
         */
        void fire(String sceneId);

        /**
         * Group id string.
         *
         * @return the string
         */
        String groupId();

        int role();
    }
}
