package com.aliyun.iot.homelink.demo.page.scene.list;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.helper.ItemTouchHelper;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.Toast;

import com.aliyun.iot.aep.component.router.Router;
import com.aliyun.iot.homelink.demo.page.scene.BaseLazyFragment;
import com.aliyun.iot.homelink.demo.page.scene.R;
import com.aliyun.iot.homelink.demo.page.scene.api.RouterHelper;
import com.aliyun.iot.homelink.demo.page.scene.api.ScenesResponse;
import com.aliyun.iot.homelink.demo.page.scene.entities.Order;
import com.aliyun.iot.homelink.demo.page.scene.entities.Scene;
import com.aliyun.iot.homelink.demo.page.scene.log.LogListActivity;
import com.aliyun.iot.homelink.demo.page.scene.widgets.BetterViewAnimator;
import com.aliyun.iot.homelink.demo.page.scene.widgets.CircleView;
import com.aliyun.iot.link.ui.component.LinkToast;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.resource.drawable.DrawableTransitionOptions;
import com.bumptech.glide.request.RequestOptions;
import com.chad.library.adapter.base.BaseQuickAdapter;
import com.chad.library.adapter.base.callback.ItemDragAndSwipeCallback;
import com.chad.library.adapter.base.listener.OnItemDragListener;

import java.util.ArrayList;
import java.util.List;


import static com.aliyun.iot.homelink.demo.page.scene.Utils.checkNotNull;


/**
 * @author sinyuk
 * @date 2018/6/14
 */
@SuppressWarnings("unused")
public class SceneListFragment extends BaseLazyFragment implements SceneListContract.View {
    public static final String TAG = "SceneList";
    public static final int REQUEST_CODE_EDIT_SCENE = 0x01;
    public static final int REQUEST_CODE_INIT_SCENE = 0x02;
    private int pageNo = 1;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.scene_list_view, container, false);
    }

    private ImageView backdrop;
    private View addButton;

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        backdrop = view.findViewById(R.id.backdrop);
        recyclerView = view.findViewById(R.id.recycleView);
        addButton = view.findViewById(R.id.add);
        swipeRefreshLayout = view.findViewById(R.id.refreshLayout);
        setupActionBar(view);
        setupRecyclerView();
    }

    @Override
    protected void lazyDo() {
        sceneListPresenter.start();
    }

    private RecyclerView recyclerView;
    private SwipeRefreshLayout swipeRefreshLayout;
    private SceneAdapter adapter;
    private boolean isFiringScene = false;
    private View header;

    private void setupRecyclerView() {
        LinearLayoutManager layoutManager = new LinearLayoutManager(getContext());
        recyclerView.setLayoutManager(layoutManager);
        assert getContext() != null;
        recyclerView.addItemDecoration(
                new MarginDecoration(R.dimen.scene_list_item_margin, getContext()));
        adapter = new SceneAdapter();

        adapter.setHeaderAndEmpty(false);
        header = View.inflate(getContext(), R.layout.scene_list_header, null);
        adapter.addHeaderView(header);

        recyclerView.setAdapter(adapter);
        View empty = View.inflate(getContext(), R.layout.scene_list_empty, null);
        adapter.setEmptyView(empty);
        adapter.disableLoadMoreIfNotFullPage(recyclerView);
        adapter.setOnLoadMoreListener(new BaseQuickAdapter.RequestLoadMoreListener() {
            @Override
            public void onLoadMoreRequested() {
                sceneListPresenter.loadScenes(pageNo);
            }
        }, recyclerView);
        adapter.setOnItemClickListener(onItemClickListener);
        adapter.setOnItemChildClickListener(onItemChildClickListener);
        adapter.setOnItemDragListener(onItemDragListener);
        swipeRefreshLayout.setEnabled(true);
        swipeRefreshLayout.setColorSchemeColors(getResources().getColor(R.color.colorAccent));
        swipeRefreshLayout.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                refresh();
            }
        });
    }

    private void refresh() {
        pageNo = 1;
        sceneListPresenter.loadScenes(pageNo);
    }

    @Override
    public void initScenes(ScenesResponse data) {
        pageNo = 2;
        adapter.setNewData(data.scenes);
//        adapter.setEnableLoadMore(data.total > data.pageSize);
    }

    @Override
    public void loadMoreScenes(ScenesResponse data) {
        if (data.scenes == null || data.scenes.isEmpty()) {
            adapter.loadMoreEnd(true);
        } else {
            int count = adapter.getData().size() + data.scenes.size();
            adapter.addData(data.scenes);
            if (data.total > count) {
                adapter.loadMoreComplete();
                pageNo++;
            } else {
                adapter.loadMoreEnd(true);
            }
        }
    }

    @Override
    public void loadMoreFailed(String message) {
        adapter.loadMoreFail();
    }

    @Override
    public void refreshFailed(String message) {
        swipeRefreshLayout.setRefreshing(false);
        LinkToast.makeText(getContext(), message, Toast.LENGTH_SHORT).show();
    }


    @Override
    public void hideRefresh() {
        swipeRefreshLayout.setRefreshing(false);
    }

    @Override
    public void showRefresh() {
        swipeRefreshLayout.setRefreshing(true);
    }

    @Override
    public void showBackdrop(@SuppressWarnings("unused") String url) {
        Glide.with(this)
                .load(R.drawable.ic_backdrop)
                .apply(new RequestOptions().centerCrop())
                .transition(new DrawableTransitionOptions().crossFade())
                .into(backdrop);
    }

    @Override
    public void reorderSucceed() {
        toggleOrderMode(false);
    }

    @Override
    public void reorderFailed(String message) {
        LinkToast.makeText(getContext(), message, Toast.LENGTH_SHORT).show();
    }

    @Override
    public void onFireFailed(String sceneId, String message) {
        LinkToast.makeText(getContext(), message, Toast.LENGTH_SHORT).show();
        isFiringScene = false;
    }

    @Override
    public void onFireSucceed(final String sceneId) {
        int dataPosition = RecyclerView.NO_POSITION;
        for (int i = 0; i < adapter.getData().size(); i++) {
            //noinspection ConstantConditions
            if (adapter.getItem(i) != null &&
                    adapter.getItem(i).id.equals(sceneId)) {
                dataPosition = i;
                break;
            }
        }
        if (dataPosition != RecyclerView.NO_POSITION) {
            final int realPosition = dataPosition + adapter.getHeaderLayoutCount();
            Scene scene = adapter.getItem(dataPosition);
            CircleView view = (CircleView) adapter
                    .getViewByPosition(recyclerView, realPosition, R.id.circleView);
            if (view != null) {
                final int finalPosition = dataPosition;
                view.setCallback(new CircleView.AnimateCallback() {
                    @SuppressWarnings("ConstantConditions")
                    @Override
                    public void onStop() {
                        adapter.getItem(finalPosition).status = SceneItemViewHolder.ON;
                        adapter.getItem(finalPosition).enable = true;
                        adapter.notifyDataSetChanged();
                    }
                });
                view.execute();
            }
        }
        isFiringScene = false;
    }

    @Override
    public void loadHouseFailed(String message) {
        LinkToast.makeText(getContext(), message, Toast.LENGTH_SHORT).show();
    }

    @Override
    public void updateRole(int role) {
        switch (role) {
            case 1: {
                ItemDragAndSwipeCallback itemDragAndSwipeCallback =
                        new ItemDragAndSwipeCallback(adapter);
                ItemTouchHelper itemTouchHelper = new ItemTouchHelper(itemDragAndSwipeCallback);
                itemTouchHelper.attachToRecyclerView(recyclerView);
                adapter.enableDragItem(itemTouchHelper, R.id.list_item, true);
                if (addButton != null) {
                    addButton.setVisibility(View.VISIBLE);
                }
                break;
            }
            case 2:
            default: {
                if (addButton != null) {
                    addButton.setVisibility(View.GONE);
                }
                adapter.disableDragItem();
                break;
            }
        }
    }

    private SceneListContract.Presenter sceneListPresenter;

    @Override
    public void setPresenter(SceneListContract.Presenter presenter) {
        this.sceneListPresenter = presenter;
    }

    private boolean inOrderMode = false;
    private List<Scene> sceneListCopy = null;

    private void toggleOrderMode(boolean open) {
        if (open == inOrderMode) {
            return;
        }
        inOrderMode = open;
        swipeRefreshLayout.setEnabled(!inOrderMode);
        adapter.setEnableLoadMore(!open);
        swipeRefreshLayout.setEnabled(!open);
        if (open) {
            adapter.removeHeaderView(header);
            viewAnimator.setDisplayedChildId(R.id.orderBar);
        } else {
            if (sceneListCopy != null) {
                sceneListCopy.clear();
            }
            adapter.setHeaderView(header);
            viewAnimator.setDisplayedChildId(R.id.titleBar);
        }
    }

    private int startPosition = RecyclerView.NO_POSITION;
    private final OnItemDragListener onItemDragListener = new OnItemDragListener() {
        @Override
        public void onItemDragStart(RecyclerView.ViewHolder viewHolder, int pos) {
            Log.d("OnItemDragListener", "onItemDragStart: " + pos);
            if (swipeRefreshLayout.isEnabled()) {
                swipeRefreshLayout.setEnabled(false);
            }
            startPosition = pos;
            if (!inOrderMode) {
                if (sceneListCopy == null) {
                    sceneListCopy = new ArrayList<>();
                } else {
                    sceneListCopy.clear();
                }
                sceneListCopy.addAll(adapter.getData());
            }
        }

        @Override
        public void onItemDragMoving(RecyclerView.ViewHolder source, int from,
                                     RecyclerView.ViewHolder target, int to) {

        }

        @Override
        public void onItemDragEnd(RecyclerView.ViewHolder viewHolder, int pos) {
            Log.d("OnItemDragListener", "onItemDragEnd: " + pos);
            if (startPosition != RecyclerView.NO_POSITION && startPosition != pos) {
                if (!inOrderMode) {
                    toggleOrderMode(true);
                }
            }
        }
    };

    private final BaseQuickAdapter.OnItemClickListener onItemClickListener = new BaseQuickAdapter.OnItemClickListener() {
        @Override
        public void onItemClick(BaseQuickAdapter a, View v, int p) {
            if (inOrderMode) {
                return;
            }
            Scene scene = adapter.getItem(p);
            if (scene == null) {
                return;
            }
            Bundle bundle = new Bundle();
            bundle.putString("sceneType", "hc");
            bundle.putString("sceneId", scene.id);
            bundle.putString("groupId", sceneListPresenter.groupId());
            Router.getInstance().toUrlForResult(getActivity(),
                    RouterHelper.EDIT_SCENE,
                    REQUEST_CODE_EDIT_SCENE,
                    bundle);
        }
    };

    private final BaseQuickAdapter.OnItemChildClickListener onItemChildClickListener = new BaseQuickAdapter.OnItemChildClickListener() {
        @Override
        public void onItemChildClick(BaseQuickAdapter a, View v, int adapterPosition) {
            if (R.id.circleView == v.getId()) {
                checkNotNull(sceneListPresenter);
                if (inOrderMode) {
                    return;
                }
                if (isFiringScene) {
                    LinkToast.makeText(getContext(), R.string.hint_scene_being_firing,
                            Toast.LENGTH_SHORT).show();
                } else {
                    Scene scene = adapter.getItem(adapterPosition);
                    checkNotNull(scene);
                    if (!scene.enable) {
                        LinkToast.makeText(getContext(), R.string.hint_scene_disable,
                                Toast.LENGTH_SHORT).show();
                        return;
                    }

                    if (scene.status != SceneItemViewHolder.ON) {
                        LinkToast.makeText(getContext(), R.string.hint_scene_offline,
                                Toast.LENGTH_SHORT).show();
                        return;
                    }

                    sceneListPresenter.fire(scene.id);
                    isFiringScene = true;
                }
            }
        }
    };

    private BetterViewAnimator viewAnimator;

    private void setupActionBar(View view) {
        view.findViewById(R.id.log).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                checkNotNull(sceneListPresenter);
                if (sceneListPresenter.groupId() != null) {
                    LogListActivity.start(getContext(), sceneListPresenter.groupId());
                } else {
                    LinkToast.makeText(getContext(), "没有房屋数据", Toast.LENGTH_SHORT).show();
                }
            }
        });

        addButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (sceneListPresenter.role() == 1) {
                    if (inOrderMode) {
                        // Reordering, quickly return
                        return;
                    }
                    Bundle bundle = new Bundle();
                    bundle.putString("sceneType", "hc");
                    bundle.putString("groupId", sceneListPresenter.groupId());
                    Router.getInstance().toUrlForResult(getActivity(),
                            RouterHelper.EDIT_SCENE,
                            REQUEST_CODE_INIT_SCENE,
                            bundle);
                } else {
                    LinkToast.makeText(getContext(), "家庭成员无权限操作", Toast.LENGTH_SHORT).show();
                }

            }
        });
        view.findViewById(R.id.cancel)
                .setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        checkNotNull(sceneListCopy);
                        adapter.setNewData(new ArrayList<>(sceneListCopy));
                        toggleOrderMode(false);
                    }
                });

        viewAnimator = view.findViewById(R.id.viewAnimator);
        view.findViewById(R.id.save).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                List<Order> orders = getOrderList(sceneListCopy, adapter.getData());
                if (orders.isEmpty()) {
                    toggleOrderMode(false);
                } else {
                    sceneListPresenter.reorder(orders);
                }
            }
        });
    }

    public List<Order> getOrderList(List<Scene> originList, List<Scene> newList) {
        List<Order> orderList = new ArrayList<>();
        for (Scene o1 : originList) {
            for (Scene o2 : newList) {
                if (o1.equals(o2) && originList.indexOf(o1) != newList.indexOf(o2)) {
                    Order order = new Order();
                    order.fromOrder = originList.indexOf(o1);
                    order.toOrder = newList.indexOf(o2);
                    order.sceneId = o1.id;
                    orderList.add(order);
                }
            }
        }
        return orderList;
    }


}