package com.aliyun.iot.homelink.demo.page.scene.log;

import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import com.aliyun.iot.homelink.demo.page.scene.R;
import com.aliyun.iot.homelink.demo.page.scene.api.LogsResponse;
import com.aliyun.iot.homelink.demo.page.scene.details.DetailsActivity;
import com.aliyun.iot.homelink.demo.page.scene.entities.Log;
import com.aliyun.iot.homelink.demo.page.scene.widgets.InsetDividerDecoration;
import com.aliyun.iot.link.ui.component.LinkToast;
import com.aliyun.iot.link.ui.component.RefreshRecycleViewLayout;
import com.aliyun.iot.link.ui.component.simpleLoadview.LinkSimpleLoadView;
import com.chad.library.adapter.base.BaseQuickAdapter;

import static com.aliyun.iot.homelink.demo.page.scene.Utils.checkNotNull;

/**
 * @author sinyuk
 * @date 2018/6/14
 */
public class LogListFragment extends Fragment implements LogListContract.View {


    private int pageNo = 1;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.log_list_view, container, false);
    }

    private RecyclerView recyclerView;
    private SwipeRefreshLayout swipeRefreshLayout;

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        recyclerView = view.findViewById(R.id.recycleView);
        swipeRefreshLayout = view.findViewById(R.id.refreshLayout);

        setupActionBar(view);
        setupRecyclerView();
        logListPresenter.start();
    }

    private void setupActionBar(View view) {
        view.findViewById(R.id.back).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (getActivity() != null) {
                    getActivity().finish();
                }
            }
        });
    }

    private LogAdapter adapter;

    private void setupRecyclerView() {
        final LinearLayoutManager layoutManager = new LinearLayoutManager(getContext());
        recyclerView.setLayoutManager(layoutManager);
        adapter = new LogAdapter();
        recyclerView.setHasFixedSize(true);
        recyclerView.setAdapter(adapter);
        View empty = View.inflate(getContext(), R.layout.empty_layout, null);
        adapter.setEmptyView(empty);
        adapter.setOnItemClickListener(new BaseQuickAdapter.OnItemClickListener() {
            @Override
            public void onItemClick(BaseQuickAdapter a, View v, int p) {
                Log log = adapter.getItem(p);
                if (log != null && log.result == 0) {
                    checkNotNull(logListPresenter);
                    DetailsActivity.start(getContext(),
                            logListPresenter.groupId(),
                            log.id,
                            log.sceneId,
                            log.time);
                }
            }
        });
        SceneLogListDecoration decoration = new SceneLogListDecoration(getContext(),
                new SceneLogListDecoration.DecorationCallback() {
                    @Override
                    public String getTimeStr(int position) {
                        if (position != -1 && adapter.getItem(position) != null) {
                            //noinspection ConstantConditions
                            return adapter.getItem(position).getHeadWeekday();
                        }
                        return "";
                    }

                    @Override
                    public boolean isFirstInGroup(int position) {
                        if (position == -1) {
                            return false;
                        }
                        if (position == 0) {
                            return true;
                        } else {
                            String last = adapter.getItem(position - 1).getHeadWeekday();
                            String current = adapter.getItem(position).getHeadWeekday();
                            return current != null && !current.equals(last);
                        }
                    }
                });
        recyclerView.addItemDecoration(decoration);
        adapter.setEnableLoadMore(false);
        adapter.setOnLoadMoreListener(new BaseQuickAdapter.RequestLoadMoreListener() {
            @Override
            public void onLoadMoreRequested() {
                logListPresenter.loadLogs(pageNo);
            }
        }, recyclerView);

        swipeRefreshLayout.setColorSchemeColors(getResources().getColor(R.color.colorAccent));
        swipeRefreshLayout.setOnRefreshListener(new SwipeRefreshLayout.OnRefreshListener() {
            @Override
            public void onRefresh() {
                pageNo = 1;
                logListPresenter.updateTimestamp(System.currentTimeMillis());
            }
        });
    }

    @Override
    public void initLogs(LogsResponse data) {
        pageNo++;
        adapter.setNewData(data.logs);
        adapter.setEnableLoadMore(data.total > data.pageSize);
    }

    @Override
    public void loadMoreLogs(LogsResponse data) {
        if (data.logs == null || data.logs.isEmpty()) {
            adapter.loadMoreEnd(true);
        } else {
            if (data.total > (adapter.getItemCount() + data.logs.size())) {
                adapter.loadMoreComplete();
                pageNo++;
            } else {
                adapter.loadMoreEnd(true);
            }
            adapter.addData(data.logs);
        }
    }

    @Override
    public void loadMoreFailed(String message) {
        adapter.loadMoreFail();
    }

    @Override
    public void refreshFailed(String message) {
        swipeRefreshLayout.setRefreshing(false);
        LinkToast.makeText(getContext(), message, Toast.LENGTH_SHORT).show();
    }

    @Override
    public void hideRefresh() {
        swipeRefreshLayout.setRefreshing(false);
    }

    @Override
    public void showRefresh() {
        swipeRefreshLayout.setRefreshing(true);
    }

    private LogListContract.Presenter logListPresenter;

    @Override
    public void setPresenter(LogListContract.Presenter presenter) {
        logListPresenter = presenter;
    }
}
