package com.aliyun.iot.homelink.demo.page.scene.log;

import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.aliyun.iot.homelink.demo.page.scene.api.LogsResponse;
import com.aliyun.iot.homelink.demo.page.scene.api.ResponseCallback;
import com.aliyun.iot.homelink.demo.page.scene.data.SceneRepository;

import static com.aliyun.iot.homelink.demo.page.scene.Utils.checkNotNull;

/**
 * @author sinyuk
 * @date 2018/6/14
 */
public class LogListPresenter implements LogListContract.Presenter {

    private static final int PAGE_SIZE = 10;
    private final LogListContract.View logListView;
    private final SceneRepository sceneRepository;
    private String groupId;
    private long nowTime;

    LogListPresenter(@NonNull SceneRepository repository,
                     @NonNull LogListContract.View view,
                     @NonNull String groupId,
                     long nowTime) {
        checkNotNull(groupId);
        this.groupId = groupId;
        this.nowTime = nowTime;
        logListView = view;
        sceneRepository = repository;
        logListView.setPresenter(this);
    }

    @Override
    public void start() {
        loadLogs(1);
    }

    @Override
    public void loadLogs(final int pageNo) {
        if (pageNo == 1) {
            logListView.showRefresh();
        }
        sceneRepository.requestLogs(pageNo, PAGE_SIZE, nowTime, groupId, new ResponseCallback<LogsResponse>() {
            @Override
            public void onFail(String message) {
                if (1 == pageNo) {
                    logListView.refreshFailed(message);
                } else {
                    logListView.loadMoreFailed(message);
                }
            }

            @Override
            public void onSuccess(LogsResponse data) {
                if (1 == pageNo) {
                    logListView.hideRefresh();
                    logListView.initLogs(data);
                } else {
                    logListView.loadMoreLogs(data);
                }
            }
        });
    }

    @Override
    public void updateTimestamp(long time) {
        nowTime = time;
        loadLogs(1);
    }

    @Override
    public String groupId() {
        return groupId;
    }
}
