package com.aliyun.iot.homelink.demo.page.share.data;

import android.annotation.SuppressLint;
import android.support.annotation.VisibleForTesting;

import com.google.gson.annotations.SerializedName;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @author sinyuk
 */
public class RoomResponse {
    public String qrCode;
    public RoomShareContent content;

    public static class RoomShareContent {

        @SerializedName("sceneCount")
        int sceneCount;
        @SerializedName("roomInfo")
        List<RoomInfoBean> roomInfo;

        static class RoomInfoBean {

            @SerializedName("roomName")
            public String roomName;
            @SerializedName("deviceCount")
            public int deviceCount;
        }
    }

    private static final int MAX_VISIBLE_COUNT = 6;

    @SuppressLint("DefaultLocale")
    public static List<String[]> convertToMap(RoomShareContent content) throws IndexOutOfBoundsException {
        List<String[]> result = new ArrayList<>();
        LinkedHashMap<String, String> map = new LinkedHashMap<>();
        if (content == null) {
            return result;
        }
        if (content.roomInfo != null && !content.roomInfo.isEmpty()) {
            int realCount = content.roomInfo.size();
            int visibleCount = realCount > MAX_VISIBLE_COUNT ? 5 : realCount;
            for (int i = 0; i < visibleCount; i++) {
                RoomShareContent.RoomInfoBean item = content.roomInfo.get(i);
                if (item != null) {
                    map.put(item.roomName, item.deviceCount + "台设备");
                }
            }

            if (realCount > MAX_VISIBLE_COUNT) {
                int leftRoom = content.roomInfo.size() - (MAX_VISIBLE_COUNT - 1);
                int leftDevices = 0;
                for (int i = MAX_VISIBLE_COUNT - 1; i < content.roomInfo.size(); i++) {
                    leftDevices += content.roomInfo.get(i).deviceCount;
                }
                map.put(String.format("等%d个房间", leftRoom), leftDevices + "台设备");
            }
        }
        map.put("场景", content.sceneCount + "个");
        for (Map.Entry<String, String> entry : map.entrySet()) {
            String[] strings = new String[2];
            strings[0] = entry.getKey();
            strings[1] = entry.getValue();
            result.add(strings);
        }
        return result;
    }

    @SuppressWarnings("unused")
    @VisibleForTesting
    public static RoomResponse mock(int count) {
        RoomResponse response = new RoomResponse();
        RoomShareContent content = new RoomShareContent();
        List<RoomShareContent.RoomInfoBean> roomInfoBeanList = new ArrayList<>();
        for (int i = 1; i <= count; i++) {
            RoomShareContent.RoomInfoBean bean = new RoomShareContent.RoomInfoBean();
            bean.roomName = "房间" + i;
            bean.deviceCount = i;
            roomInfoBeanList.add(bean);
        }
        content.roomInfo = roomInfoBeanList;
        content.sceneCount = 10;
        response.content = content;
        response.qrCode = "https://www.taobao.com";
        return response;
    }
}