package com.aliyun.iot.homelink.demo.page.share.ui;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.media.MediaScannerConnection;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;


import com.aliyun.iot.aep.sdk.threadpool.ThreadPool;
import com.aliyun.iot.homelink.demo.page.R;
import com.aliyun.iot.homelink.demo.page.share.bottomsheet.BottomPopUpDialog;
import com.aliyun.iot.homelink.demo.page.share.data.DeviceResponse;
import com.aliyun.iot.homelink.demo.page.share.data.RoomResponse;
import com.aliyun.iot.link.ui.component.LinkToast;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.DataSource;
import com.bumptech.glide.load.engine.GlideException;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.Target;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.qrcode.QRCodeWriter;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;


import static com.aliyun.iot.homelink.demo.page.share.ui.ShareViewPresenter.ACTION_ROOM_SHARE;

/**
 * @author sinyuk
 * @date 2018/6/26
 */
public class ShareFragment extends Fragment implements ShareViewContract.View {

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.share_fragment, container, false);
    }


    private RecyclerView recyclerView;
    private ImageView codeView;
    private View screenShot;
    private TextView saveButton;
    private MediaScannerConnection scannerConnection;

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        recyclerView = view.findViewById(R.id.recyclerView);
        codeView = view.findViewById(R.id.qrcode);
        screenShot = view.findViewById(R.id.screenShot);
        TextView title = view.findViewById(R.id.title);
        if (ACTION_ROOM_SHARE.equals(presenter.bizType())) {
            title.setText(R.string.label_share_room);
        } else {
            title.setText(R.string.label_share_house);
        }

        view.findViewById(R.id.back).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (getActivity() != null) {
                    getActivity().finish();
                }
            }
        });

        saveButton = view.findViewById(R.id.save);
        saveButton.setEnabled(false);
        saveButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                showBottomSheet();
            }
        });

        setupRecyclerView();
        presenter.start();
    }

    void showBottomSheet() {
        new BottomPopUpDialog.Builder()
                .setDialogData(getResources().getStringArray(R.array.bottom_sheet_array))
                .setCallBackDismiss(true)
                .setItemTextColor(0, R.color.color_title)
                .setItemOnListener(new BottomPopUpDialog.BottomPopDialogOnClickListener() {
                    @Override
                    public void onDialogClick(String tag) {
                        if (!getString(R.string.cancel).equals(tag)) {
                            onSaveScreenShot();
                        }
                    }
                })
                .show(getChildFragmentManager(), BottomPopUpDialog.class.getSimpleName());
    }


    private void setupRecyclerView() {
        GridLayoutManager layoutManager = new GridLayoutManager(getContext(), 2) {
            @Override
            public boolean canScrollVertically() {
                return false;
            }

            @Override
            public boolean canScrollHorizontally() {
                return false;
            }
        };
        recyclerView.setHasFixedSize(true);
        recyclerView.setLayoutManager(layoutManager);
        recyclerView.setHasFixedSize(true);
        recyclerView.setNestedScrollingEnabled(false);
    }


    @SuppressLint("SetTextI18n")
    private View getHeaderView() {
        View header = View.inflate(getContext(), R.layout.list_header, null);
        if (ShareViewPresenter.ACTION_ROOM_SHARE.equals(presenter.bizType())) {
//            ((TextView) header.findViewById(R.id.name)).setText(presenter.roomName());
            header.findViewById(R.id.hint).setVisibility(View.INVISIBLE);
        } else {
            header.findViewById(R.id.hint).setVisibility(View.VISIBLE);
        }
        ((TextView) header.findViewById(R.id.name)).setText(presenter.houseName());
        return header;
    }


    @Override
    public void initRooms(List<String[]> data) {
        HouseShareAdapter adapter = new HouseShareAdapter(data);
        adapter.addHeaderView(getHeaderView());
        if (recyclerView.getAdapter() == null) {
            recyclerView.setAdapter(adapter);
        } else {
            recyclerView.swapAdapter(adapter, true);
        }
    }

    @SuppressLint("DefaultLocale")
    @Override
    public void initDevices(List<DeviceResponse.Device> contents, int total) {
        if (contents != null && contents.size() > DeviceResponse.MAX_VISIBLE_COUNT) {
            contents = contents.subList(0, 10);
        }
        DeviceShareAdapter adapter = new DeviceShareAdapter(contents, getContext());
        adapter.addHeaderView(getHeaderView());
        if (DeviceResponse.MAX_VISIBLE_COUNT < total) {
            View footer = View.inflate(getContext(), R.layout.device_list_footer, null);
            ((TextView) footer.findViewById(R.id.count)).setText(
                    String.format("等%d台设备", total - DeviceResponse.MAX_VISIBLE_COUNT));
            adapter.addFooterView(footer);
        }
        View header = View.inflate(getContext(), R.layout.device_list_header, null);
        ((TextView) header.findViewById(R.id.room)).setText(presenter.roomName());
        adapter.addHeaderView(header);
        adapter.setHeaderFooterEmpty(false, false);
        if (recyclerView.getAdapter() == null) {
            recyclerView.setAdapter(adapter);
        } else {
            recyclerView.swapAdapter(adapter, true);
        }
    }

    @Override
    public void createQRImage(String content) {
        final int widthPix = codeView.getWidth();
        final int heightPix = codeView.getHeight();
        Bitmap bitmap;
        try {
            if (TextUtils.isEmpty(content)) {
                return;
            }
            Map<EncodeHintType, Object> hints = new LinkedHashMap<>();
            hints.put(EncodeHintType.CHARACTER_SET, "utf-8");
            hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.H);
            //设置空白边距的宽度
            hints.put(EncodeHintType.MARGIN, 1);
            BitMatrix bitMatrix = new QRCodeWriter().encode(content, BarcodeFormat.QR_CODE, widthPix, heightPix, hints);
            int[] pixels = new int[widthPix * heightPix];
            for (int y = 0; y < heightPix; y++) {
                for (int x = 0; x < widthPix; x++) {
                    if (bitMatrix.get(x, y)) {
                        pixels[y * widthPix + x] = 0xff000000;
                    } else {
                        pixels[y * widthPix + x] = 0xffffffff;
                    }
                }
            }
            bitmap = Bitmap.createBitmap(widthPix, heightPix, Bitmap.Config.ARGB_8888);
            bitmap.setPixels(pixels, 0, widthPix, 0, 0, widthPix, heightPix);
            loadQRCode(bitmap);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void initDataFailed(Exception e) {
        if (e != null) {
            LinkToast.makeText(getContext(), e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }


    @Override
    public void initDevicesFailed(Exception e) {
        if (e != null) {
            LinkToast.makeText(getContext(), e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }

    private void loadQRCode(Bitmap bitmap) {
        Glide.with(getContext()).asDrawable()
                .load(bitmap)
                .listener(new RequestListener<Drawable>() {
                    @Override
                    public boolean onLoadFailed(@Nullable GlideException e, Object m, Target<Drawable> t, boolean i) {
                        if (e != null) {
                            LinkToast.makeText(getContext(), e.getMessage(), Toast.LENGTH_SHORT).show();
                        }
                        return false;
                    }

                    @Override
                    public boolean onResourceReady(Drawable r, Object m, Target<Drawable> target, DataSource d, boolean i) {
                        saveButton.setEnabled(true);
                        return false;
                    }
                })
                .into(codeView);
    }

    private void onSaveScreenShot() {
        Log.d("This", "onSaveScreenShot: ");
        if (screenShot == null) {
            LinkToast.makeText(getContext(), "保存失败", Toast.LENGTH_SHORT).show();
            return;
        }
        saveButton.setEnabled(false);
        Bitmap bitmap = Bitmap.createBitmap(screenShot.getWidth(), screenShot.getHeight(),
                Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        screenShot.draw(canvas);
        try {
            boolean isHasSDCard = Environment.getExternalStorageState().equals(
                    android.os.Environment.MEDIA_MOUNTED);
            if (isHasSDCard) {
                File sdRoot = Environment.getExternalStorageDirectory();
                final String fileName = generateFileName();
                File file = new File(sdRoot, fileName);
                FileOutputStream fos = new FileOutputStream(file);
                boolean succeed = bitmap.compress(Bitmap.CompressFormat.JPEG, 70, fos);
                fos.flush();
                fos.close();
                if (!succeed) {
                    LinkToast.makeText(getContext(), "图片压缩失败", Toast.LENGTH_SHORT).show();
                    return;
                }
                MediaStore.Images.Media.insertImage(
                        getContext().getContentResolver(),
                        file.getAbsolutePath(),
                        fileName, null);
                Intent intent = new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE);
                Uri uri = Uri.fromFile(file);
                intent.setData(uri);
                getContext().sendBroadcast(intent);
                scannerConnection = new MediaScannerConnection(getContext(),
                        new MediaScannerConnection.MediaScannerConnectionClient() {
                            @Override
                            public void onMediaScannerConnected() {
                                scannerConnection.scanFile(fileName, "image/*");
                            }

                            @Override
                            public void onScanCompleted(String path, Uri uri) {
                                scannerConnection.disconnect();
                                final String location;
                                if (null != uri) {
                                    location = "图片已保存到系统相册：" + uri.toString();
                                } else {
                                    location = "图片已保存到系统相册";
                                }
                                ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                                    @Override
                                    public void run() {
                                        LinkToast.makeText(getContext(), location,
                                                Toast.LENGTH_SHORT).show();
                                    }
                                });
                            }
                        });
                scannerConnection.connect();
            } else {
                LinkToast.makeText(getContext(), "未发现SD卡", Toast.LENGTH_SHORT).show();
            }
        } catch (IOException e) {
            e.printStackTrace();
            LinkToast.makeText(getContext(), "保存失败", Toast.LENGTH_SHORT).show();
        }
        saveButton.setEnabled(true);
    }

    private String generateFileName() {
        SimpleDateFormat format = new SimpleDateFormat("MMddHHmmss", Locale.getDefault());
        String dateStr = format.format(new Date(System.currentTimeMillis()));
        return dateStr + ".jpg";
    }

    private ShareViewContract.Presenter presenter;

    @Override
    public void setPresenter(ShareViewContract.Presenter presenter) {
        this.presenter = presenter;
    }
}
