package com.aliyun.iot.homelink.demo.page.share.ui;

import com.aliyun.iot.homelink.demo.page.share.BasePresenter;
import com.aliyun.iot.homelink.demo.page.share.BaseView;
import com.aliyun.iot.homelink.demo.page.share.data.DeviceResponse;
import com.aliyun.iot.homelink.demo.page.share.data.RoomResponse;

import java.util.List;
import java.util.Map;

/**
 * The interface Share view contract.
 *
 * @author sinyuk
 * @date 2018 /7/10
 */
public interface ShareViewContract {
    /**
     * The interface View.
     */
    interface View extends BaseView<Presenter> {

        /**
         * Init rooms.
         *
         * @param contents the contents
         */
        void initRooms(List<String[]> contents);

        /**
         * Init devices.
         *
         * @param contents the contents
         * @param total    the total
         */
        void initDevices(List<DeviceResponse.Device> contents, int total);

        /**
         * Create qr image.
         *
         * @param content the content
         */
        void createQRImage(String content);

        /**
         * Init data failed.
         *
         * @param e the e
         */
        void initDataFailed(Exception e);

        /**
         * Init devices failed.
         *
         * @param e the e
         */
        void initDevicesFailed(Exception e);

    }

    /**
     * The interface Presenter.
     */
    interface Presenter extends BasePresenter {

        /**
         * Init data.
         */
        void initData();

        /**
         * Load devices.
         */
        void loadDevices();

        /**
         * Biz type string.
         *
         * @return the string
         */
        String bizType();

        /**
         * Group id string.
         *
         * @return the string
         */
        String groupId();

        /**
         * Room id string.
         *
         * @return the string
         */
        String roomId();

        /**
         * House name string.
         *
         * @return the string
         */
        String houseName();

        /**
         * Room name string.
         *
         * @return the string
         */
        String roomName();
    }
}
