package com.aliyun.iot.homelink.demo.page.share.ui;

import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.alibaba.fastjson.JSONArray;
import com.aliyun.iot.aep.sdk.apiclient.IoTAPIClient;
import com.aliyun.iot.aep.sdk.apiclient.IoTAPIClientFactory;
import com.aliyun.iot.aep.sdk.apiclient.callback.IoTCallback;
import com.aliyun.iot.aep.sdk.apiclient.callback.IoTResponse;
import com.aliyun.iot.aep.sdk.apiclient.request.IoTRequest;
import com.aliyun.iot.aep.sdk.apiclient.request.IoTRequestBuilder;
import com.aliyun.iot.aep.sdk.threadpool.ThreadPool;
import com.aliyun.iot.homelink.demo.page.share.data.DeviceResponse;
import com.aliyun.iot.homelink.demo.page.share.data.RoomResponse;
import com.aliyun.iot.homelink.demo.page.share.data.ShareResponse;
import com.google.gson.Gson;

import java.util.List;

/**
 * @author sinyuk
 * @date 2018/7/10
 */
public class ShareViewPresenter implements ShareViewContract.Presenter {

    public static final String ACTION_HOUSE_SHARE = "HOUSE_SHARE";
    public static final String ACTION_ROOM_SHARE = "ROOM_SHARE";
    private final String bizType;
    private final String groupId;
    private final String roomId;
    private final String houseName;
    private final String roomName;
    private final ShareViewContract.View shareView;
    private final IoTAPIClient client;
    private final Gson gson;

    ShareViewPresenter(
            @NonNull String bizType,
            @NonNull String groupId,
            @Nullable String roomId,
            @Nullable String houseName,
            @Nullable String roomName,
            ShareViewContract.View view) {
        client = new IoTAPIClientFactory().getClient();
        gson = new Gson();
        this.bizType = bizType;
        this.groupId = groupId;
        this.roomId = roomId;
        this.houseName = houseName;
        this.roomName = roomName;
        shareView = view;
        shareView.setPresenter(this);
    }

    @Override
    public void initData() {
        String apiVersion = "1.0.3";
        JSONArray houseIds = new JSONArray();
        houseIds.add(groupId);
        IoTRequestBuilder builder = new IoTRequestBuilder()
                .setAuthType("iotAuth")
                .setPath("/homelink/share/generate/qrcode")
                .setApiVersion(apiVersion)
                .addParam("bizType", bizType)
                .addParam("houseIds", houseIds);

        if (ACTION_ROOM_SHARE.equals(bizType)) {
            JSONArray roomIds = new JSONArray();
            roomIds.add(roomId);
            builder.addParam("roomIds", roomIds);
            loadDevices();
        }
        client.send(builder.build(), new IoTCallback() {
            @Override
            public void onFailure(IoTRequest ioTRequest, final Exception e) {
                ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                    @Override
                    public void run() {
                        shareView.initDataFailed(e);
                    }
                });
            }

            @Override
            public void onResponse(IoTRequest request, final IoTResponse response) {
                if (200 != response.getCode()) {
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            shareView.initDataFailed(new Exception(response.getLocalizedMsg()));
                        }
                    });
                } else {
                    if (ACTION_HOUSE_SHARE.equals(bizType)) {
                        final RoomResponse data =
                                gson.fromJson(response.getData().toString(), RoomResponse.class);
                        final List<String[]> listData = RoomResponse.convertToMap(data.content);
                        ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                            @Override
                            public void run() {
                                shareView.createQRImage(data.qrCode);
                                shareView.initRooms(listData);
                            }
                        });
                    } else if (ACTION_ROOM_SHARE.equals(bizType)) {
                        final ShareResponse data =
                                gson.fromJson(response.getData().toString(), ShareResponse.class);
                        ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                            @Override
                            public void run() {
                                shareView.createQRImage(data.qrCode);
                            }
                        });
                    } else {
                        shareView.initDataFailed(new Exception("分享失败"));
                    }
                }
            }
        });
    }

    @Override
    public void loadDevices() {
        IoTRequestBuilder builder = new IoTRequestBuilder()
                .setAuthType("iotAuth")
                .setPath("/homelink/room/device/list")
                .setApiVersion("1.0.0")
                .addParam("pageNo", 1)
                .addParam("pageSize", DeviceResponse.MAX_VISIBLE_COUNT)
                .addParam("roomId", roomId);
        client.send(builder.build(), new IoTCallback() {
            @Override
            public void onFailure(IoTRequest ioTRequest, final Exception e) {
                ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                    @Override
                    public void run() {
                        shareView.initDevicesFailed(e);
                    }
                });
            }

            @Override
            public void onResponse(final IoTRequest request, final IoTResponse response) {
                if (200 != response.getCode()) {
                    ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                        @Override
                        public void run() {
                            shareView.initDevicesFailed(new Exception(response.getLocalizedMsg()));
                        }
                    });
                } else {
                    try {
                        final DeviceResponse data =
                                gson.fromJson(response.getData().toString(), DeviceResponse.class);
                        ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
                            @Override
                            public void run() {
                                shareView.initDevices(data.data, data.total);
                            }
                        });

                    } catch (Exception e) {
                        e.printStackTrace();
                        shareView.initDevicesFailed(e);
                    }
                }
            }
        });
    }

    @Override
    public String bizType() {
        return bizType;
    }

    @Override
    public String groupId() {
        return groupId;
    }

    @Override
    public String roomId() {
        return roomId;
    }

    @Override
    public String houseName() {
        return houseName;
    }

    @Override
    public String roomName() {
        return roomName;
    }


    @Override
    public void start() {
        initData();
    }
}
